const test = require('tape')
const { assoc } = require('ramda')
const { createStore, applyMiddleware } = require('redux')

const { promiseMiddleware, createReducer, createActionHandler, createActionCreator, combineActionHandlers } = require('..')

const actionHandler = combineActionHandlers(
  createActionHandler('ADD_ASYNC', (state, action) => state),
  createActionHandler('ADD', (state, action) => assoc('value', action.payload, state)),
  createActionHandler('ERROR', (state, action) => assoc('error', action.payload, state))
)

const reducer = createReducer({value: 0, error: false}, actionHandler)

const addAsync = createActionCreator((value) => ({
  type: 'ADD_ASYNC',
  payload: new Promise((resolve, reject) => {
    setTimeout(() => typeof value === 'number' ? resolve(add(value)) : reject(error()), 800)
  })
}))

const add = createActionCreator((value) => ({ type: 'ADD', payload: value }))

const error = createActionCreator(() => ({ type: 'ERROR', payload: true }))

const createNewStore = () => createStore(reducer, {value: 0, error: false}, applyMiddleware(promiseMiddleware))

test('promiseMiddleware should be a function', (t) => {
  t.plan(1)
  t.equal(typeof promiseMiddleware, 'function')
})

test('promiseMiddleware should dispatch success action on resolve', (t) => {
  t.plan(1)
  const store = createNewStore()
  store.subscribe(() => store.getState().value === 1 && t.pass())
  store.dispatch(addAsync(1))
})

test('promiseMiddleware should dispatch fail action on catch', (t) => {
  t.plan(1)
  const store = createNewStore()
  store.subscribe(() => store.getState().error !== false && t.pass())
  store.dispatch(addAsync('cow'))
})

test('promiseMiddleware shoudnt interfere with non-promise payload actions', (t) => {
  t.plan(1)
  const store = createNewStore()
  store.subscribe(() => store.getState().value === 1 && t.pass())
  store.dispatch(add(1))
})

